/**
 * 
 */
package com.rapidtransform.core.meta.service;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.rapidtransform.core.meta.Dictionary;
import com.rapidtransform.core.meta.MetaException;
import com.rapidtransform.core.meta.MetaModel;
import com.rapidtransform.core.meta.MetaProperty;
import com.rapidtransform.core.meta.MetaType;
import com.rapidtransform.core.model.namespace.Namespace;
import com.rapidtransform.core.model.namespace.QName;

/**
 * @author Steve Werner
 *
 */
public class DictionaryImpl implements Dictionary {
	
	private static Log log = LogFactory.getLog(DictionaryImpl.class);
	
	private Map<QName, MetaModel> loadedModels;
	private Map<QName, MetaType> loadedTypes;
	private Map<String, MetaProperty> loadedProperties;
	private Map<Namespace, QName> namespaceQnames = new HashMap<Namespace, QName>();
	// this map associates each Meta Type with it's defined namespace
	private Map<QName, Namespace> typeNamespaceMap = new HashMap<QName, Namespace>();
	private List<MetaType> searchableTypes = new ArrayList<MetaType>();;
	
	
	public DictionaryImpl() {
		loadedModels = new HashMap<QName, MetaModel>();
		loadedTypes = new HashMap<QName, MetaType>();
		loadedProperties = new HashMap<String, MetaProperty>();
	}

	/**
	 * @return Returns the loadedModels.
	 */
	public Map<QName, MetaModel> getLoadedModels() {
		return loadedModels;
	}

	/**
	 * @param loadedModels The loadedModels to set.
	 */
	public void setLoadedModels(Map<QName, MetaModel> loadedModels) {
		this.loadedModels = loadedModels;
	}

	/* (non-Javadoc)
	 * @see com.rapidtransform.core.meta.Dictionary#addModel(com.rapidtransform.core.meta.MetaModel)
	 */
	public void addModel(MetaModel model) {
		loadedModels.put(model.getQname(), model);
		
		// add all the types defined in the model to the qname/type map
		for (MetaType type : model.getTypes()) {
			loadedTypes.put(type.getName(), type);
			String typeName = type.getName().toString();
			for (MetaProperty prop : type.getProperties()) {
				if (loadedProperties.containsKey(typeName+":"+prop.getName())) {
					log.warn("Two properties are mapped to the same name "+typeName+":"+prop.getName());
				}
				loadedProperties.put(typeName+":"+prop.getName(), prop);
			}
			
			if (type.isSearchable()) {
				searchableTypes.add(type);
			}
			
			for (Namespace namespace : model.getNamespaces()) {
				namespaceQnames.put(namespace, model.getQname());
				
				for (MetaType typeInNamespace : model.getTypes()) {
					typeNamespaceMap.put(typeInNamespace.getName(), namespace);
					
				}
			}
		}
		
	}

	/* (non-Javadoc)
	 * @see com.rapidtransform.core.meta.Dictionary#removeModel(com.rapidtransform.core.model.namespace.QName)
	 */
	public void removeModel(QName qName) throws MetaException {
		MetaModel model = loadedModels.remove(qName);
		if (model == null) {
			throw new MetaException("MetaModel not found for QName "+qName);
		}
	}

	/* (non-Javadoc)
	 * @see com.rapidtransform.core.meta.Dictionary#getModel(com.rapidtransform.core.model.namespace.QName)
	 */
	public MetaModel getModel(QName qName) {
		return loadedModels.get(qName);
	}

	/* (non-Javadoc)
	 * @see com.rapidtransform.core.meta.Dictionary#getType(com.rapidtransform.core.model.namespace.QName)
	 */
	public MetaType getType(QName qName) {
		return loadedTypes.get(qName);
	}

	/**
	 * Returns the MetaProperty given the internal representation of the property name, which consists of the 
	 * MetaType prefix, MetaType localName, and propertyName strings concatenated together and separate by _.
	 * 
	 * For example, for MetaType prd:product and property foo, the internal property name is:
	 * prd_product_foo
	 * 
	 * @return Returns the loadedProperties.
	 */
	public MetaProperty getProperty(String propertyName) {
		return loadedProperties.get(propertyName);
	}

	/* (non-Javadoc)
	 * @see com.rapidtransform.core.meta.Dictionary#getQName(com.rapidtransform.core.model.namespace.Namespace)
	 */
	public QName getQName(Namespace namespace) {
		return namespaceQnames.get(namespace);
	}

	/* (non-Javadoc)
	 * @see com.rapidtransform.core.meta.Dictionary#getNamespace(com.rapidtransform.core.model.namespace.QName)
	 */
	public Namespace getNamespace(QName type) {
		return typeNamespaceMap.get(type);
	}

	/* (non-Javadoc)
	 * @see com.rapidtransform.core.meta.Dictionary#getSearchableTypes()
	 */
	public Collection<MetaType> getSearchableTypes() {
		return Collections.unmodifiableCollection(searchableTypes);
	}
	
	
	
}
