/**
 * The contents of this file are subject to the Mozilla Public License 
 * Version 1.1 with a permitted attribution clause (the "License");
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 * 
 * http://www.rapidtransform.com/legal/license.txt
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is Convelo Open Source.
 * 
 * The Initial Developer of the Original Code is Rapid Transform, Inc.
 * Portions created by Rapid Transform are Copyright (C) 2006 Rapid Transform, Inc. 
 * All Rights Reserved.
 */
package com.rapidtransform.core.meta.service;

import java.io.IOException;
import java.io.InputStream;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jibx.runtime.BindingDirectory;
import org.jibx.runtime.IBindingFactory;
import org.jibx.runtime.IUnmarshallingContext;
import org.jibx.runtime.JiBXException;
import org.springframework.context.ResourceLoaderAware;
import org.springframework.core.io.Resource;
import org.springframework.core.io.ResourceLoader;
import org.springframework.core.io.support.PathMatchingResourcePatternResolver;
import org.springframework.core.io.support.ResourcePatternResolver;

import com.rapidtransform.core.ConveloException;
import com.rapidtransform.core.meta.Dictionary;
import com.rapidtransform.core.meta.InvalidMetaRelationException;
import com.rapidtransform.core.meta.MetaException;
import com.rapidtransform.core.meta.MetaModel;
import com.rapidtransform.core.meta.MetaProperty;
import com.rapidtransform.core.meta.MetaRelation;
import com.rapidtransform.core.meta.MetaService;
import com.rapidtransform.core.meta.MetaType;
import com.rapidtransform.core.meta.def.BaseMetaModel;
import com.rapidtransform.core.model.XMLMarshalingException;
import com.rapidtransform.core.model.namespace.Namespace;
import com.rapidtransform.core.model.namespace.QName;
import com.rapidtransform.core.model.namespace.QNameFactory;
import com.rapidtransform.core.type.DynamicPropertyHolder;
import com.rapidtransform.core.type.PropertyHolder;

/**
 * @author Steve Werner
 *
 */
public class MetaServiceImpl implements MetaService, ResourceLoaderAware {

	private final static Log log = LogFactory.getLog(MetaServiceImpl.class);
	
	private Dictionary dictionary;
	
	private ResourceLoader resourceLoader;
	
	// list of XML model definitions to load on startup
	private List<String> models;
	private List<Resource> modelResources;
		
	/**
	 * @return Returns the models.
	 */
	public List<String> getModels() {
		return models;
	}

	/**
	 * @param models The models to set.
	 */
	public void setModels(List<String> models) {
		this.models = models;
	}

	/**
	 * @return Returns the dictionary.
	 */
	public Dictionary getDictionary() {
		return dictionary;
	}

	/**
	 * @param dictionary The dictionary to set.
	 */
	public void setDictionary(Dictionary dictionary) {
		this.dictionary = dictionary;
	}

	/**
	 * @return Returns the modelResources.
	 */
	public List<Resource> getModelResources() {
		return modelResources;
	}

	/**
	 * @param modelResources The modelResources to set.
	 */
	public void setModelResources(List<Resource> modelResources) {
		this.modelResources = modelResources;
	}

	public void load() throws MetaException {
		for (String xmlModel : models) {
			log.debug("loading meta model "+xmlModel);
			if (resourceLoader == null) {
				resourceLoader = new PathMatchingResourcePatternResolver();
			}
			try {
				Resource[] resources = ((ResourcePatternResolver)resourceLoader).getResources(xmlModel);
				for (Resource resource : resources) {
					InputStream xmlInput = resource.getInputStream();//getClass().getClassLoader().getResourceAsStream(xmlModel);
					if (xmlInput == null) {
						throw new MetaException("Error loading XML Model: "+xmlModel);
					}
					MetaModel model = loadModel(xmlInput);
					log.info("Loaded MetaModel with QName "+model.getQname().toString());
					dictionary.addModel(model);
				}

			} catch (MetaException me) {
				log.error("Unable to load XML MetaModel: "+xmlModel, me);
			} catch (IOException ioe) {
				log.error("Unable to load XML MetaModel: "+xmlModel, ioe);
			}
		}
	}
	
	/* (non-Javadoc)
	 * @see com.rapidtransform.core.meta.MetaService#loadModel(com.rapidtransform.core.meta.MetaModel)
	 */
	public MetaModel loadModel(InputStream xmlModelStream) throws MetaException {
		try {
			return BaseMetaModel.fromXML(xmlModelStream);
		} catch (XMLMarshalingException xme) {
			log.error("Unable to unmarshal MetaModel from XML");
			throw new MetaException("Unable to unmarshal MetaModel from XML", xme);
		}
	}

	/* (non-Javadoc)
	 * @see com.rapidtransform.core.meta.MetaService#addModel(com.rapidtransform.core.meta.MetaModel)
	 */
	public void addModel(MetaModel model) {
		dictionary.addModel(model);
	}

	/* (non-Javadoc)
	 * @see com.rapidtransform.core.meta.MetaService#removeModel(com.rapidtransform.core.model.namespace.QName)
	 */
	public void removeModel(QName qName) {
		try {
			dictionary.removeModel(qName);
		} catch (MetaException me) {
			log.error("Meta model for QName "+qName+" not removed");
		}
	}

	/* (non-Javadoc)
	 * @see com.rapidtransform.core.meta.MetaService#getLoadedModel(com.rapidtransform.core.model.namespace.QName)
	 */
	public MetaModel getLoadedModel(QName qName) {
		return dictionary.getModel(qName);
	}

	/* (non-Javadoc)
	 * @see com.rapidtransform.core.meta.MetaService#getLoadedModel(com.rapidtransform.core.model.namespace.Namespace)
	 */
	public MetaModel getLoadedModel(Namespace namespace) throws MetaException {
		QName qname = dictionary.getQName(namespace);
		if (qname == null) {
			throw new MetaException("QName undefined for Namespace "+namespace);
		}
		return getLoadedModel(qname);
	}

	/* (non-Javadoc)
	 * @see com.rapidtransform.core.meta.MetaService#getType(com.rapidtransform.core.model.namespace.QName)
	 */
	public MetaType getType(QName qname) throws MetaException {
		return dictionary.getType(qname);
	}

	/* (non-Javadoc)
	 * @see com.rapidtransform.core.meta.MetaService#getProperties(com.rapidtransform.core.model.namespace.QName)
	 */
	public PropertyHolder getProperties(QName type) throws MetaException {
		MetaType metatype = getType(type);
		String sType = type.toString() + ":";
		DynamicPropertyHolder holder = new DynamicPropertyHolder();
		Map<String, Serializable> propMap = new HashMap<String, Serializable>(metatype.getProperties().size());
		for (MetaProperty prop : metatype.getProperties()) {
			// initialize all properties to null
			propMap.put(sType + prop.getName(), null);
		}
		//holder.setProperties(propMap);
		holder.setMetaTypeName(type);
		return holder;
	}

	/* (non-Javadoc)
	 * @see com.rapidtransform.core.meta.MetaService#getType(java.lang.String)
	 */
	public MetaType getType(String qname) throws MetaException {
		return getType(QNameFactory.createQName(qname));
	}

	/* (non-Javadoc)
	 * @see com.rapidtransform.core.meta.MetaService#getProperties(java.lang.String)
	 */
	public PropertyHolder getProperties(String typeQname) throws MetaException {
		return getProperties(QNameFactory.createQName(typeQname));
	}

	/**
	 * Finds the MetaProperty for the given internal property name.
	 * 
	 * @see com.rapidtransform.core.meta.service.DictionaryImpl#getProperty(java.lang.String)
	 */
	public MetaProperty getProperty(String fullyQualifiedPropertyName) throws MetaException {
		return dictionary.getProperty(fullyQualifiedPropertyName);
	}

	/* (non-Javadoc)
	 * @see com.rapidtransform.core.meta.MetaService#getRelationTarget(com.rapidtransform.core.model.namespace.QName, com.rapidtransform.core.model.namespace.QName)
	 */
	public QName getRelationTarget(QName source, QName relation) throws InvalidMetaRelationException, MetaException {
		MetaType metaType = getType(source);
		for (MetaRelation metaRel : metaType.getRelations()) {
			if (metaRel.getName().equals(relation)) {
				return metaRel.getTargetType();
			}
		}
		log.error("Relation "+relation+" is not valid for type "+source);
		throw new InvalidMetaRelationException("Invalid relation "+relation+" for source type "+source);
	}

	/**
	 * returns the namespace associated with the given type.  This is how types are mapped to M1 objects - for each namespace, there is one and only
	 * one M1 object type.
	 */
	public Namespace getNamespace(QName type) {
		return dictionary.getNamespace(type);
	}

	/* (non-Javadoc)
	 * @see com.rapidtransform.core.meta.MetaService#getLoadedModels()
	 */
	public Collection<MetaModel> getLoadedModels() {
		return Collections.unmodifiableCollection(dictionary.getLoadedModels().values());
	}

	/* (non-Javadoc)
	 * @see com.rapidtransform.core.meta.MetaService#getSearchableTypes()
	 */
	public Collection<MetaType> getSearchableTypes() throws MetaException {
		return dictionary.getSearchableTypes();
	}

	/* (non-Javadoc)
	 * @see org.springframework.context.ResourceLoaderAware#setResourceLoader(org.springframework.core.io.ResourceLoader)
	 */
	public void setResourceLoader(ResourceLoader loader) {
		this.resourceLoader = loader;
	}

}
