/**
 * 
 */
package com.rapidtransform.core.meta.service;

import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.HibernateException;
import org.hibernate.MappingException;
import org.hibernate.cfg.Configuration;
import org.hibernate.mapping.Column;
import org.hibernate.mapping.Component;
import org.hibernate.mapping.PersistentClass;
import org.hibernate.mapping.Property;
import org.hibernate.mapping.SimpleValue;
import org.hibernate.mapping.Table;
import org.springframework.orm.hibernate3.LocalSessionFactoryBean;

import com.rapidtransform.core.meta.MetaException;
import com.rapidtransform.core.meta.MetaModel;
import com.rapidtransform.core.meta.MetaRelation;
import com.rapidtransform.core.meta.MetaType;
import com.rapidtransform.core.meta.MetaProperty;
import com.rapidtransform.core.meta.MetaService;
import com.rapidtransform.core.model.namespace.Namespace;
import com.rapidtransform.core.type.DynamicPropertyHolder;
import com.rapidtransform.core.type.PropertyHolder;

/**
 * The MetaSessionFactory enhances the Hibernate Configuration to include definitions for the meta information
 * defined in the MetaModel.  This factory extends the Spring LocalSessionFactoryBean by adding post processing
 * on the Configuration.
 * 
 * @author Steve Werner
 *
 */
public class MetaSessionFactory extends LocalSessionFactoryBean {
	
	public static Log log = LogFactory.getLog(MetaSessionFactory.class);
	
	// A Map with Hibernate mapped entity-name as the key and Meta Namespace as the value
	private Map<String, Namespace> metaEntities;
	
	private MetaService metaService;
	
	/**
	 * @return Returns the metaEntities.
	 */
	public Map<String, Namespace> getMetaEntities() {
		return metaEntities;
	}

	/**
	 * @param metaEntities The metaEntities to set.
	 */
	public void setMetaEntities(Map<String, Namespace> metaEntities) {
		this.metaEntities = metaEntities;
	}

	/**
	 * @return Returns the metaService.
	 */
	public MetaService getMetaService() {
		return metaService;
	}

	/**
	 * @param metaService The metaService to set.
	 */
	public void setMetaService(MetaService metaService) {
		this.metaService = metaService;
	}

	/* 
	 * @see org.springframework.orm.hibernate3.LocalSessionFactoryBean#postProcessConfiguration(org.hibernate.cfg.Configuration)
	 */
	@Override
	protected void postProcessConfiguration(Configuration config) throws HibernateException {
		PersistentClass classMapping = null;
		Namespace namespace = null;
		MetaModel metaModel = null;
		String mappingName = null;
		String mappingColumn = null;
		Property mappingProp = null;
		config.buildMappings();
		for (Iterator classMapIter = config.getClassMappings(); classMapIter.hasNext(); ) {
			classMapping = (PersistentClass)classMapIter.next();
			
			// only add configuration information for PropertyHolder mappings
			if (classMapping.getClassName() != null && classMapping.getClassName().equals(DynamicPropertyHolder.class.getName())) {
				if (classMapping.getEntityName() != null) {
					namespace = metaEntities.get(classMapping.getEntityName());
					if (namespace == null) {
						log.warn("No namespace defined for entity-name "+classMapping.getEntityName());
						continue;
					}
					log.debug("Namespace for entity-name "+classMapping.getEntityName()+" is "+namespace);
					
					// find the dynamic-component property
					Component dynamicComponent = null;
					for (Iterator propIter = classMapping.getPropertyIterator(); propIter.hasNext(); ) {
						Property prop = (Property)propIter.next();
						if (prop.getValue() instanceof Component && 
								((Component)prop.getValue()).isDynamic()) {
							dynamicComponent = (Component)prop.getValue();
							break;
						}
					}
					
					// the definition must contain a dynamic-component placeholder
					if (dynamicComponent == null) {
						log.error("No dynamic-component defined in Hibernate mapping for entity "+classMapping.getEntityName());
						continue;
					}
					
					try {
						metaModel = metaService.getLoadedModel(namespace);
						for (MetaType metatype : metaModel.getTypes()) {
							for (MetaProperty metaprop : metatype.getProperties()) {
								mappingName = metatype.getName().toString() + ":" + metaprop.getName();
								mappingColumn = mappingName.replace(':', '_');
								mappingProp = new Property();
								mappingProp.setName(mappingName);
								mappingProp.setNodeName(mappingName);
								SimpleValue value = new SimpleValue(classMapping.getTable());
								MetaType propType = metaService.getType(metaprop.getType());
								Column column = new Column(mappingColumn);
								value.setTypeName(propType.getJavaClass());
								value.addColumn(column);
								mappingProp.setValue(value);
								
								// add the new property to the dynamic-component to update the class mapping
								dynamicComponent.addProperty(mappingProp);
								
								// add the new column to the Table model so the schema is updated
								classMapping.getTable().addColumn(column);
							}
						}
						log.debug("created the dynamic-component: "+dynamicComponent);
					} catch (MetaException me) {
						log.error("Error retrieving MetaModel for namespace "+namespace);
					}
				}
			}
			
		}
		super.postProcessConfiguration(config);
	}
}
