/**
 * 
 */
package com.rapidtransform.core.search.lucene;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;

import com.rapidtransform.core.model.namespace.QName;
import com.rapidtransform.core.search.AdvancedSearchCommand;
import com.rapidtransform.core.search.SearchCommand;
import com.rapidtransform.core.search.SearchResult;
import com.rapidtransform.core.search.SearchResults;
import com.rapidtransform.core.search.SearchService;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.compass.core.Compass;
import org.compass.core.CompassCallback;
import org.compass.core.CompassDetachedHits;
import org.compass.core.CompassException;
import org.compass.core.CompassHit;
import org.compass.core.CompassHits;
import org.compass.core.CompassQuery;
import org.compass.core.CompassQueryBuilder;
import org.compass.core.CompassSession;
import org.compass.core.CompassTemplate;
import org.compass.core.Property;
import org.hibernate.Query;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.transaction.PlatformTransactionManager;
import org.springframework.transaction.TransactionStatus;
import org.springframework.transaction.support.TransactionCallback;
import org.springframework.transaction.support.TransactionTemplate;

import com.rapidtransform.core.service.impl.ServiceImpl;

/**
 * @author Steve Werner
 *
 */
public class CompassSearchService extends ServiceImpl implements SearchService, InitializingBean {

    private static Log log = LogFactory.getLog(CompassSearchService.class);
    
    private Compass compass;
    private CompassTemplate compassTemplate;
    private PlatformTransactionManager transactionManager;

    public Compass getCompass() {
		return compass;
	}
	public void setCompass(Compass compass) {
		this.compass = compass;
	}
	
	public void afterPropertiesSet() throws Exception {
		if (compass == null) {
            throw new IllegalArgumentException("Must set compass property to initialize "+CompassSearchService.class.getName());
        }
        this.compassTemplate = new CompassTemplate(compass);
	}

	public CompassTemplate getCompassTemplate() {
		return compassTemplate;
	}
	
	/**
	 * Searches the Lucene index for matches to the query in the CompassSearchCommand.  If the results should
	 * be paginated, the pageSize and currentPage attributes must be set on the SearchCommand.  If pageSize and
	 * currentPage are null, then the results will not be paged.
	 * 
	 * The format for the query follows Lucene's query format.  See Lucene documentation for details.
	 * @link http://lucene.apache.org/java/docs/queryparsersyntax.html
	 * 
	 * @param searchType The QName of the Meta Type that is being searched.  If null, then all types will be searched.
	 */
	public Collection<SearchResult> search(String searchType, final SearchCommand searchCommand) {
		SearchResultsCompass searchResults = null;
		final QName searchTypeQName = new QName(searchType);

		searchResults = (SearchResultsCompass) getCompassTemplate().execute(new CompassCallback() {
    		public Object doInCompass(CompassSession session) throws CompassException {
    			return doSearch(searchTypeQName.toEncodedString(), searchCommand, session);
    		}
    	});

		ArrayList<SearchResult> results = new ArrayList<SearchResult>();
		SearchResult result = null;
	    for (CompassHit hit : searchResults.getHits()) {
	    	org.compass.core.Resource resource = hit.getResource();
	    	result = new SearchResult();
	    	for (Property prop : resource.getProperties()) {
	    		result.getData().put(prop.getName(), prop.getStringValue());
	    	}
	    	results.add(result);
	    }
	    return results;
	}

	/**
	 * Creates the search indexes.  If old search indexes exist, they are removed once the
	 * new indexes are created.  No system down time is required to re-index since the new
	 * indexes will be swapped in for the old indexes by using the index locks.	 
	 */
	public void index() {
		// TODO Auto-generated method stub
		
	}
	
	/**
	 * Performs the search on the index and returns the paginated results.
	 * 
	 * TODO Performance increase: return more than one page of results and eliminate the need to re-query the index for each page.
	 * 
	 * @param searchCommand
	 * @param session
	 * @return
	 */
    protected SearchResultsCompass doSearch(String searchAlias, SearchCommand searchCommand, CompassSession session) {
    	CompassQueryBuilder queryBuilder = session.queryBuilder();
    	CompassQuery query = getQuery(queryBuilder, searchCommand);
    	String[] aliases = {searchAlias};
        query.setAliases(aliases);
        
        log.debug("Executing search for query: "+query);
        CompassHits hits = query.hits();

        //session.queryBuilder().
        Integer pageSize = searchCommand.getPageSize();
        CompassDetachedHits detachedHits = null;
        SearchResultsCompass.Page[] pages = null;
        if (pageSize == null) {
            detachedHits = hits.detach();
        } else {
            int iPageSize = pageSize.intValue();
            int page = 0;
            int hitsLength = hits.getLength();
            if (searchCommand.getCurrentPage() != null) {
                page = searchCommand.getCurrentPage().intValue();
            }
            int from = page * iPageSize;
            if (from > hits.getLength()) {
                from = hits.getLength() - iPageSize;
                detachedHits = hits.detach(from, hitsLength);
            } else if ((from + iPageSize) > hitsLength) {
                detachedHits = hits.detach(from, hitsLength);
            } else {
                detachedHits = hits.detach(from, from + iPageSize);
            }
            int numberOfPages = (int) Math.ceil((float) hitsLength / iPageSize);
            int to = 0;
            pages = new SearchResultsCompass.Page[numberOfPages];
            for (int i = 0; i < pages.length; i++) {
                pages[i] = new SearchResultsCompass.Page();
                pages[i].setFrom(i * iPageSize + 1);
                pages[i].setSize(iPageSize);
                to = (i + 1) * iPageSize;
                if (from >= (pages[i].getFrom() - 1) && from < to) {
                    pages[i].setSelected(true);
                } else {
                    pages[i].setSelected(false);
                }
            }
            if (numberOfPages > 0) {
                SearchResultsCompass.Page lastPage = pages[numberOfPages - 1];
                lastPage.setSize(hitsLength - lastPage.getFrom());
            }
        }
        SearchResultsCompass searchResults = new SearchResultsCompass(detachedHits.getHits());
        searchResults.setPages(pages);
        return searchResults;
    }
    
    private CompassQuery getQuery(CompassQueryBuilder builder, AdvancedSearchCommand searchCommand) {
    	CompassQuery query = null;//builder.queryString(searchCommand.getQuery().queryString().trim()).toQuery();
        return query;
    }
    
    private CompassQuery getQuery(CompassQueryBuilder builder, SearchCommand searchCommand) {
    	CompassQuery query = null;
    	if (searchCommand instanceof CompassSearchCommand) {
    		query = builder.queryString(((CompassSearchCommand)searchCommand).getQuery().queryString().trim()).toQuery();
    	}
    	else if (searchCommand instanceof AdvancedSearchCommand) {
    		AdvancedSearchCommand adv = (AdvancedSearchCommand)searchCommand;
    		CompassQueryBuilder.CompassBooleanQueryBuilder boolBuilder = builder.bool();
    		Map<String, String> criteria = adv.getSearchCriteria();
    		for (String field : criteria.keySet()) {
    			if (field != null && criteria.get(field) != null) {
    				String value = criteria.get(field);
    				if (value != null) {
    					if (value instanceof String && ((String)value).length() <= 0)
    						continue;
    					boolBuilder.addMust(builder.term(field, value.toLowerCase()));
    				}
    				
    			}
    		}
    		query = boolBuilder.toQuery();
    	}
        return query;
    }

}
