/**
 * The contents of this file are subject to the Mozilla Public License 
 * Version 1.1 (the "License"); you may not use this file except in 
 * compliance with the License. You may obtain a copy of the License at 
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is Rapid PLM Open Source.
 * 
 * The Initial Developer of the Original Code is Rapid Transform, Inc.
 * Portions created by Rapid Transform are Copyright (C) 2005 Rapid Transform, Inc. 
 * All Rights Reserved.
 * 
 * Contributor(s): ______________________________________.
 */
package com.rapidtransform.core.search.lucene;

import org.compass.core.CompassHit;
import org.compass.core.CompassHits;

import com.rapidtransform.core.search.SearchResults;

/**
 * The results object returned by {@link com.rapidtransform.core.search.lucene.SearchServiceCompass} 
 * when executing a search.  This class encapsulates the hits returned from the Lucene query as well
 * as the pages if pagination is being used.
 * 
 * The search time is also include for performance monitoring.
 * 
 * @author Steve
 *
 */
public class SearchResultsCompass implements SearchResults {
    private CompassHit[] hits;
    private Page[] pages;
    private long searchTime;
	
    /**
     * Constructor using the hits returned from Compass search and time for the search to execute.
     * @param hits
     * @param time
     */
    public SearchResultsCompass(CompassHit[] hits, long time) {
    	this.hits = hits;
    	this.searchTime = time;
    }
    
    /**
     * Constructor specifing only the hits returned from the search
     * @param hits
     */
    public SearchResultsCompass(CompassHit[] hits) {
    	this.hits = hits;
    }
    
    /**
	 * @return Returns the hits.
	 */
	public CompassHit[] getHits() {
		return hits;
	}
	/**
	 * @param hits The hits to set.
	 */
	public void setHits(CompassHit[] hits) {
		this.hits = hits;
	}
	/**
	 * @return Returns the pages.
	 */
	public Page[] getPages() {
		return pages;
	}
	/**
	 * @param pages The pages to set.
	 */
	public void setPages(Page[] pages) {
		this.pages = pages;
	}
	/**
	 * @return Returns the searchTime.
	 */
	public long getSearchTime() {
		return searchTime;
	}
	/**
	 * @param searchTime The searchTime to set.
	 */
	public void setSearchTime(long searchTime) {
		this.searchTime = searchTime;
	}
	
	/**
	 * SearchResultsCompass internal class used for pagination.  For each page, this class
	 * holds the page's from index and number of results (size) for the page.  It also includes
	 * a flag indicating if this is the currently selected page.
	 * 
	 * A Page can cache it's hits by setting it's internal hits array.  This will improve performance
	 * since the search engine will not be required to re-execute the query for each page.
	 */
	public static class Page {
		private int from;
		private int size;
		private boolean selected = false;
		private boolean loaded = false;
		private CompassHits[] hits;
		
		/**
		 * Default constructor
		 */
		public Page() {
			
		}
		
		/**
		 * Constructor used to create a page with it's hits cached internally.
		 * @param hits
		 */
		public Page(CompassHits[] hits) {
			this.setHits(hits);
		}
		
		/**
		 * @return Returns the from.
		 */
		public int getFrom() {
			return from;
		}
		/**
		 * @param from The from to set.
		 */
		public void setFrom(int from) {
			this.from = from;
		}
		/**
		 * @return Returns the selected.
		 */
		public boolean isSelected() {
			return selected;
		}
		/**
		 * @param selected The selected to set.
		 */
		public void setSelected(boolean selected) {
			this.selected = selected;
		}
		/**
		 * @return Returns the size.
		 */
		public int getSize() {
			return size;
		}
		/**
		 * @param size The size to set.
		 */
		public void setSize(int size) {
			this.size = size;
		}
		/**
		 * Used to check the loaded status of hits cached in this page.
		 * @return Returns whether or not this Page has it's hits loaded.
		 */
		public boolean isLoaded() {
			return loaded;
		}
		/**
		 * Returns the cached hits for this page or null if no hits are loaded.  Check loaded flag to determine
		 * loaded status.
		 * 
		 * @return Returns the hits.
		 */
		public CompassHits[] getHits() {
			return hits;
		}
		/**
		 * Set the Hits for this page.  These hits act as a cache for pagination so the search engine doesn't
		 * have to re-execute the query for each page.
		 * 
		 * @param hits The hits to set.
		 */
		public void setHits(CompassHits[] hits) {
			this.hits = hits;
			this.loaded = true;
		}		

	}
    
}
