/**
 * The contents of this file are subject to the Mozilla Public License 
 * Version 1.1 (the "License"); you may not use this file except in 
 * compliance with the License. You may obtain a copy of the License at 
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is Rapid PLM Open Source.
 * 
 * The Initial Developer of the Original Code is Rapid Transform, Inc.
 * Portions created by Rapid Transform are Copyright (C) 2005 Rapid Transform, Inc. 
 * All Rights Reserved.
 * 
 * Contributor(s): ______________________________________.
 */
package com.rapidtransform.core.service.impl;

import java.util.Collection;
import java.util.HashMap;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.rapidtransform.core.ConveloException;
import com.rapidtransform.core.dao.ValidValueDAO;
import com.rapidtransform.core.model.ValidValue;
import com.rapidtransform.core.service.ValidValueService;

/**
 * @author Steve
 *
 */
public class ValidValueServiceImpl extends ServiceImpl implements ValidValueService {
	
	private Log log = LogFactory.getLog(ValidValueServiceImpl.class);
	
	/**
	 * Map of DAOs for each of the ValidValue subclass types.  The key for the map
	 * is the class name of the subclass and the value is the DAO for the subclass.
	 * 
	 */
	private HashMap<String, ValidValueDAO> validValueDAOs;
	
	public HashMap<String, ValidValueDAO> getValidValueDAOs() {
		return validValueDAOs;
	}

	public void setValidValueDAOs(HashMap<String, ValidValueDAO> validValueDAOs) {
		this.validValueDAOs = validValueDAOs;
	}


	/* (non-Javadoc)
	 * @see com.rapidtransform.core.service.ValidValueService#save(com.rapidtransform.core.model.ValidValue)
	 * @throws RapidException Exception generated if subclass is not mapped in the validValueDAOs map
	 */
	public ValidValue save(ValidValue validValue) throws ConveloException {
		// get the DAO from the map and use that DAO to perform the save operation
		ValidValueDAO dao = validValueDAOs.get(validValue.getClass().getName());
		if (dao == null) {
			for (String c : validValueDAOs.keySet()) {
				log.debug("Valid Value DAO key: "+c);
			}
			throw new ConveloException("Valid value list does not exist for type "+validValue.getClass().getName());
		}
		return (ValidValue)dao.save(validValue);
	}

	/**
	 * Returns the ValidValue object for the given subclass and id.
	 * 
	 * @param id
	 * @return
	 */
	public ValidValue find(Class subclass, Long id) throws ConveloException {
		// get the DAO from the map and use that DAO to perform the save operation
		ValidValueDAO dao = validValueDAOs.get(subclass.getName());
		if (dao == null) {
			for (String c : validValueDAOs.keySet()) {
				log.debug("Valid Value DAO key: "+c);
			}
			throw new ConveloException("Valid value list does not exist for type "+subclass.getName());
		}
		return (ValidValue)dao.findById(id);	
	}
	
	/**
	 * Returns a list of all the valid values for the given ValidValue subclass
	 * @param subclass
	 * @throws ConveloException Exception generated if subclass is not mapped in the validValueDAOs map
	 */
	public Collection<ValidValue> findAll(Class subclass) throws ConveloException {
		// get the DAO from the map and use it to find all objects for the given subclass
		ValidValueDAO dao = validValueDAOs.get(subclass.getName());
		if (dao == null) {
			throw new ConveloException("Valid value list does not exist for type "+subclass.getName());
		}
		return (Collection<ValidValue>)dao.findAll();
	}

	/**
	 * Returns a collection of all enabled ValidValues for the given ValidValue subclass.
	 * 
	 * @param subclass
	 * @return
	 * @throws ConveloException
	 */
	public Collection<ValidValue> findEnabled(Class subclass) throws ConveloException {
		// get the DAO from the map and use it to find all objects for the given subclass
		ValidValueDAO dao = validValueDAOs.get(subclass.getName());
		if (dao == null) {
			throw new ConveloException("Valid value list does not exist for type "+subclass.getName());
		}
		return dao.findEnabled();
	}
	
	/**
	 * Returns a collection of existing value list types.
	 * @return Collection of ValidValue class names
	 */
	public Collection<String> getValueListTypes() {
		return validValueDAOs.keySet();
	}

	/**
	 * Removes the given valid value from the database.
	 * 
	 * @param validValue
	 * @exception ConveloException Thrown if ValidValue type is not mapped in the validValueDAOs map
	 */
	public void remove(ValidValue validValue) throws ConveloException {
		// get the DAO from the map and use it to find all objects for the given subclass
		ValidValueDAO dao = validValueDAOs.get(validValue.getClass().getName());
		if (dao == null) {
			throw new ConveloException("Valid value list does not exist for type "+validValue.getClass().getName());
		}
		dao.remove(validValue);
	}

}
