/**
 * The contents of this file are subject to the Mozilla Public License 
 * Version 1.1 (the "License"); you may not use this file except in 
 * compliance with the License. You may obtain a copy of the License at 
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is Rapid PLM Open Source.
 * 
 * The Initial Developer of the Original Code is Rapid Transform, Inc.
 * Portions created by Rapid Transform are Copyright (C) 2005 Rapid Transform, Inc. 
 * All Rights Reserved.
 * 
 * Contributor(s): ______________________________________.
 */
package com.rapidtransform.part.model;

import java.util.Collection;
import java.util.HashSet;
import java.util.Set;

import com.rapidtransform.classification.model.Classified;
import com.rapidtransform.core.model.VersionControlled;
import com.rapidtransform.core.util.EqualsUtil;
import com.rapidtransform.doc.model.Document;
import com.rapidtransform.org.model.Alias;
import com.rapidtransform.org.model.OrganizationElement;
import com.rapidtransform.org.model.OrganizationalData;
import com.rapidtransform.spec.model.Effective;

/**
 * An item is either a single object or an assembly of related objects.
 * 
 * The item can be classified as either a part, raw material, tool or assebly using the
 * SpecificItemClassification object.
 *   
 * @author Steve Werner
 *
 */
public class Item extends VersionControlled implements Classified, OrganizationElement, OrganizationalData, Effective {
	
	private static final long serialVersionUID = 1L;
	
	// There must be at least one itemVersion for each item
	private Set<ItemVersion> itemVersions = new HashSet<ItemVersion>();
	private Set<Alias> aliases;
	private Set<Document> documents;
	
	/**
	 * Default constructor
	 */
	public Item() {
		
	}
	
	/**
	 * Constructs an Item with the given name and creates the first version for the
	 * list of ItemVersions.
	 * 
	 * @param name
	 */
	public Item(String name) {
		//this.name = name;
		createNewVersion();
	}
	
	/**
	 * @return Returns the alias.
	 */
	public Collection<Alias> getAliases() {
		return aliases;
	}
	/**
	 * @param alias The alias to set.
	 */
	public void setAliases(Set<Alias> aliases) {
		this.aliases = aliases;
	}
	/**
	 * @return Returns the documents.
	 */
	public Collection<Document> getDocuments() {
		return documents;
	}
	/**
	 * @param documents The documents to set.
	 */
	public void setDocuments(Set<Document> documents) {
		this.documents = documents;
	}
	/**
	 * @return Returns the itemVersions.
	 */
	public Collection<ItemVersion> getItemVersions() {
		return itemVersions;
	}
	/**
	 * @param itemVersions The itemVersions to set.
	 */
	public void setItemVersions(Set<ItemVersion> itemVersions) {
		this.itemVersions = itemVersions;
	}
	
	/**
	 * Adds a new ItemVersion to the list of ItemVersions for this Item
	 * @param newVersion
	 */
	public void addItemVersion(ItemVersion newVersion) {
		itemVersions.add(newVersion);
	}
	
	/**
	 * Creates new version of this Item, adds it to the ItemVersions list, and returns
	 * the new ItemVersion object.  The ItemVersion object will automatically have this Item 
	 * set as it's parent Item.
	 * 
	 * @return ItemVersion The new item version.
	 */
	public ItemVersion createNewVersion() {
		ItemVersion newVersion = null;
		newVersion = new ItemVersion(itemVersions.size() + 1);
		newVersion.setItem(this);
		itemVersions.add(newVersion);
		return newVersion;
	}
	
	/**
	 * Returns the concept of a latest version out of the list of ItemVersions for this Item.
	 * The latest version is defined as the last version that was created for this Item - that is, the
	 * ItemVersion with the highest versionId.
	 * 
	 * @return Latest ItemVersion
	 */
	public ItemVersion getLatestVersion() {
		ItemVersion latest = null;
		// iterate through the item versions and find the one with the highest versionId, indicating it was the last one created
		for (ItemVersion iv : itemVersions) {
			if (latest == null) {
				latest = iv;
			}
			else if (iv.getVersionId() > latest.getVersionId()){
				latest = iv;
			}
		}
		return latest;
	}
	
	/**
	 * override equals since this object could be used in a Collection
	 */
    public boolean equals(Object other) {
        if (this == other) return true;
        if ( !(other instanceof Item) ) return false;

        final Item item = (Item) other;
        
        return (EqualsUtil.areEqual(item.getUuid(), getUuid()) &&
        		EqualsUtil.areEqual(item.getId(), getId()));
    }

    /**
     * Override hashCode since this object could be used in a Collection
     */
    public int hashCode() {
    	return (getUuid()  == null ? 15 : getUuid().hashCode()) ^ 
        		(getId() == null ? 29 : getId().hashCode());
    }
	
}
