/**
 * The contents of this file are subject to the Mozilla Public License 
 * Version 1.1 (the "License"); you may not use this file except in 
 * compliance with the License. You may obtain a copy of the License at 
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is Rapid PLM Open Source.
 * 
 * The Initial Developer of the Original Code is Rapid Transform, Inc.
 * Portions created by Rapid Transform are Copyright (C) 2005 Rapid Transform, Inc. 
 * All Rights Reserved.
 * 
 * Contributor(s): ______________________________________.
 */
package com.rapidtransform.part.model;

import java.util.Collection;
import java.util.HashSet;
import java.util.Set;

import com.rapidtransform.classification.model.Classified;
import com.rapidtransform.core.model.Relation;
import com.rapidtransform.core.model.VersionControlled;
import com.rapidtransform.core.type.PropertyHolder;
import com.rapidtransform.core.util.EqualsUtil;
import com.rapidtransform.doc.model.Document;
import com.rapidtransform.doc.model.Documented;
import com.rapidtransform.org.model.Alias;
import com.rapidtransform.org.model.Approvable;
import com.rapidtransform.org.model.OrganizationElement;
import com.rapidtransform.org.model.OrganizationalData;
import com.rapidtransform.product.model.ProductDesign;
import com.rapidtransform.spec.model.Effective;
import com.rapidtransform.spec.model.Specified;

/**
 * An ItemVersion is a specific version of an Item.  It contains all the data that comprises the
 * physically makeable item for each application context.
 * 
 * @author Steve Werner
 *
 */
public class ItemVersion extends VersionControlled 
	implements Classified, Approvable, OrganizationElement, OrganizationalData, Effective, Specified, Documented {
	
	/**
	 * VersionId identifies this version in relation to the other versions for a specific Item.
	 * VersionId is required.
	 */
	private Integer versionId;
	
	private Item item;
	private ProductDesign productDesign;
	private Set<ItemView> itemViews = new HashSet<ItemView>();  // specifies the item views for this version
	private Set<Alias> aliases;

	private Set<Relation> specs;
	private Set<Relation> documents;
	
	
	public ItemVersion() {
		item = new Item();
	}
	
	/**
	 * Creates a version with the specified versionId.
	 * @param versionId
	 */
	public ItemVersion(Integer versionNumber) {
		this.versionId = versionNumber;
	}
	
	/**
	 * @return Returns the alias.
	 */
	public Collection<Alias> getAliases() {
		return aliases;
	}
	/**
	 * @param alias The alias to set.
	 */
	public void setAliases(Set<Alias> aliases) {
		this.aliases = aliases;
	}
	/**
	 * @return Returns the itemViews.
	 */
	public Collection<ItemView> getItemViews() {
		return itemViews;
	}
	/**
	 * @param itemViews The itemViews to set.
	 */
	public void setItemViews(Set<ItemView> itemViews) {
		this.itemViews = itemViews;
	}
	
	public void addItemView(ItemView itemView) {
		this.itemViews.add(itemView);
	}
	
	/**
	 * @return Returns the productDesign.
	 */
	public ProductDesign getProductDesign() {
		return productDesign;
	}
	/**
	 * @param productDesign The productDesign to set.
	 */
	public void setProductDesign(ProductDesign productDesign) {
		this.productDesign = productDesign;
	}
	/**
	 * @return Returns the item.
	 */
	public Item getItem() {
		return item;
	}
	/**
	 * @param item The item to set.
	 */
	public void setItem(Item item) {
		this.item = item;
	}

    /**
	 * @return Returns the versionId.
	 */
	public Integer getVersionId() {
		return versionId;
	}

	/**
	 * @param versionId The versionId to set.
	 */
	public void setVersionId(Integer versionNumber) {
		this.versionId = versionNumber;
	}

	/**
	 * @return Returns the documents.
	 */
	public Set<Relation> getDocuments() {
		return documents;
	}

	/**
	 * @param documents The documents to set.
	 */
	public void setDocuments(Set<Relation> documents) {
		this.documents = documents;
	}

	/**
	 * @return Returns the specs.
	 */
	public Set<Relation> getSpecs() {
		return specs;
	}

	/**
	 * @param specs The specs to set.
	 */
	public void setSpecs(Set<Relation> specs) {
		this.specs = specs;
	}

	public boolean equals(Object other) {
        if (this == other) return true;
        if ( !(other instanceof ItemVersion) ) return false;

        final ItemVersion itemVersion = (ItemVersion) other;
        
        return (EqualsUtil.areEqual(itemVersion.getVersionId(), getVersionId()) &&
        		EqualsUtil.areEqual(itemVersion.getId(), getId()));
    }

    public int hashCode() {
    	return (item  == null ? 15 : item.hashCode()) ^ 
		(versionId == null ? 29 : versionId.hashCode());
    }

}
