/**
 * The contents of this file are subject to the Mozilla Public License 
 * Version 1.1 (the "License"); you may not use this file except in 
 * compliance with the License. You may obtain a copy of the License at 
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is Rapid PLM Open Source.
 * 
 * The Initial Developer of the Original Code is Rapid Transform, Inc.
 * Portions created by Rapid Transform are Copyright (C) 2005 Rapid Transform, Inc. 
 * All Rights Reserved.
 * 
 * Contributor(s): ______________________________________.
 */
package com.rapidtransform.part.model;

import java.util.Collection;
import java.util.HashSet;
import java.util.Set;
import com.rapidtransform.classification.model.Classified;
import com.rapidtransform.core.model.VersionControlled;
import com.rapidtransform.core.util.EqualsUtil;
import com.rapidtransform.doc.model.Document;
import com.rapidtransform.org.model.Alias;
import com.rapidtransform.org.model.Approvable;
import com.rapidtransform.org.model.OrganizationElement;
import com.rapidtransform.org.model.OrganizationalData;
import com.rapidtransform.product.model.FinalDefinition;
import com.rapidtransform.product.model.ItemFunctionAssociation;
import com.rapidtransform.product.model.PhysicalInstanceDefinition;

/**
 * This class represents a view of a specific item.  In the STEP standard, this is referred to as
 * the design discipline item definition.
 * 
 * The view is relevant for one or more application contexts and contains the product data for a
 * item version. 
 * 
 * @author Steve
 *
 */
public class ItemView extends VersionControlled 
	implements FinalDefinition, ItemInformation, Classified, Approvable, OrganizationElement, OrganizationalData,
	PhysicalInstanceDefinition {
	
	/**
	 * The name for this item view is required.
	 */
	private String name;
	
	/**
	 * The ItemInstances that implement this ItemView
	 */
	private Set<ItemInstance> itemInstances;
	
	/**
	 * Links this ItemView with other ItemViews.  This is used to create MakeFrom and Replaced relationships, as well as
	 * a relationship with a Tool or a general relation to another ItemView.
	 */
	private Set<ItemViewRelation> itemViewRelations;
	
	/**
	 * A set of documents describing this ItemView
	 */
	private Set<Document> documents;
	
	/**
	 * Relates this ItemView with a product function.
	 */
	private Set<ItemFunctionAssociation> itemFunctions;
	
	/**
	 * The aliases specify how this item is identified outside the immediate organization - that is, how a 
	 * partner/manufacturer identifies the item.
	 */
	private Set<Alias> aliases;  // STEP alias identification
	
	/**
	 * This relation is used for linking an ItemView to other ItemInstances that do not implement this ItemView.
	 * If this ItemView is an Assembly, then this relation will specify the subassembly children.  If this is a
	 * Collection, then the relation links the ItemView to other items that are used for the finished product, but
	 * are not assembled together.
	 */
	private Set<ItemViewInstanceRelation> viewInstanceRelations;
	
	/**
	 * The initial context for this item specifies the applicable state and domain where this ItemView
	 * can initially be used in the item/product definition process.
	 */
	private ApplicationContext initialContext;
	
	/**
	 * Additional contexts can be added for this ItemView if it can be used in other states or application
	 * domains in addition to the initial context.
	 */
	private Set<ApplicationContext> additionalContexts;
	
	/**
	 * An association back to the ItemVersion from which this ItemView is created.
	 */
	private ItemVersion itemVersion;
	
	// TODO association to custom attributes object
	
	/**
	 * Default constructor
	 */
	public ItemView() {
		
	}
	
	/**
	 * Constructs an ItemView with the given ItemVersion as it's parent.  The parameters specify
	 * the required attributes for this class.
	 * 
	 * @param name
	 * @param fromVersion
	 * @param initialContext
	 */
	public ItemView(String name, ItemVersion fromVersion, ApplicationContext initialContext) {
		this.name = name;
		this.itemVersion = fromVersion;
		this.initialContext = initialContext;
	}
	
	/**
	 * Additional contexts can be added for this ItemView if it can be used in other states or application
	 * domains in addition to the initial context.
	 * 
	 * @return Returns the additionalContexts.
	 */
	public Collection<ApplicationContext> getAdditionalContexts() {
		return additionalContexts;
	}
	/**
	 * @param additionalContexts The additionalContexts to set.
	 */
	public void setAdditionalContexts(Set<ApplicationContext> additionalContexts) {
		this.additionalContexts = additionalContexts;
	}
	
	/**
	 * Adds an application context to the set of additional contexts.
	 * @param context
	 */
	public void addApplicationContext(ApplicationContext context) {
		if (this.additionalContexts == null) {
			this.additionalContexts = new HashSet<ApplicationContext>();
		}
		this.additionalContexts.add(context);
	}
	/**
	 * The aliases specify how this item is identified outside the immediate organization - that is, how a 
	 * partner/manufacturer identifies the item.
	 * 
	 * @return Returns the alias.
	 */
	public Collection<Alias> getAliases() {
		return aliases;
	}
	/**
	 * @param alias The alias to set.
	 */
	public void setAliases(Set<Alias> aliases) {
		this.aliases = aliases;
	}
	
	/**
	 * Adds an alias to the set of aliases
	 * @param alias
	 */
	public void addAlias(Alias alias) {
		if (this.aliases == null) {
			this.aliases = new HashSet<Alias>();
		}
		aliases.add(alias);
	}
	
	/**
	 * @return Returns the documents.
	 */
	public Collection<Document> getDocuments() {
		return documents;
	}
	/**
	 * @param documents The documents to set.
	 */
	public void setDocuments(Set<Document> documents) {
		this.documents = documents;
	}
	
	/**
	 * Adds a document to the set of documents.
	 * @param document
	 */
	public void addDocument(Document document) {
		if (this.documents == null) {
			this.documents = new HashSet<Document>();
		}
		documents.add(document);
	}
	/**
	 * The initial context for this item specifies the applicable state and domain where this ItemView
	 * can initially be used in the item/product definition process.
	 * 
	 * @return Returns the initialContext.
	 */
	public ApplicationContext getInitialContext() {
		return initialContext;
	}
	/**
	 * @param initialContext The initialContext to set.
	 */
	public void setInitialContext(ApplicationContext initialContext) {
		this.initialContext = initialContext;
	}
	/**
	 * Relates this ItemView with a product function.
	 * 
	 * @return Returns the itemFunctions.
	 */
	public Collection<ItemFunctionAssociation> getItemFunctions() {
		return itemFunctions;
	}
	/**
	 * @param itemFunctions The itemFunctions to set.
	 */
	public void setItemFunctions(Set<ItemFunctionAssociation> itemFunctions) {
		this.itemFunctions = itemFunctions;
	}
	/**
	 * The ItemInstances that implement this ItemView
	 * 
	 * @return Returns the itemInstances.
	 */
	public Collection<ItemInstance> getItemInstances() {
		return itemInstances;
	}
	/**
	 * @param itemInstances The itemInstances to set.
	 */
	public void setItemInstances(Set<ItemInstance> itemInstances) {
		this.itemInstances = itemInstances;
	}
	
	/**
	 * Adds another item instances to the set of instances that implement this view.
	 */
	public void addItemInstance(ItemInstance instance) {
		if (this.itemInstances == null) {
			this.itemInstances = new HashSet<ItemInstance>();
		}
		this.itemInstances.add(instance);
	}
	
	/**
	 * Links this ItemView with other ItemViews.  This is used to create MakeFrom and Replaced relationships, as well as
	 * a relationship with a Tool or a general relation to another ItemView.
	 * 
	 * @return Returns the itemViewRelation.
	 */
	public Collection<ItemViewRelation> getItemViewRelations() {
		return itemViewRelations;
	}
	/**
	 * @param itemViewRelation The itemViewRelation to set.
	 */
	public void setItemViewRelations(Set<ItemViewRelation> itemViewRelations) {
		this.itemViewRelations = itemViewRelations;
	}
	/**
	 * @return Returns the name.
	 */
	public String getName() {
		return name;
	}
	/**
	 * @param name The name to set.
	 */
	public void setName(String name) {
		this.name = name;
	}
	/**
	 * This relation is used for linking an ItemView to other ItemInstances that do not implement this ItemView.
	 * If this ItemView is an Assembly, then this relation will specify the subassembly children.  If this is a
	 * Collection, then the relation links the ItemView to other items that are used for the finished product, but
	 * are not assembled together.
	 * 
	 * @return Returns the viewInstanceRelations.
	 */
	public Collection<ItemViewInstanceRelation> getViewInstanceRelations() {
		return viewInstanceRelations;
	}
	/**
	 * @param viewInstanceRelations The viewInstanceRelation to set.
	 */
	public void setViewInstanceRelations(
			Set<ItemViewInstanceRelation> viewInstanceRelations) {
		this.viewInstanceRelations = viewInstanceRelations;
	}

    /**
     * An association back to the ItemVersion from which this ItemView is created.
     * 
	 * @return Returns the itemVersion.
	 */
	public ItemVersion getItemVersion() {
		return itemVersion;
	}
	/**
	 * @param itemVersion The itemVersion to set.
	 */
	public void setItemVersion(ItemVersion itemVersion) {
		this.itemVersion = itemVersion;
	}
	public boolean equals(Object other) {
        if (this == other) return true;
        if ( !(other instanceof ItemView) ) return false;

        final ItemView itemView = (ItemView) other;

        return (EqualsUtil.areEqual(itemView.getName(), getName()) &&
        		EqualsUtil.areEqual(itemView.getId(), getId()));
    }

    public int hashCode() {
    	int result = getItemVersion().hashCode() + getName().hashCode();
    	if (getId() != null) {
    		result = result + getId().hashCode();
    	}
        return result;
    }		
}
