/**
 * The contents of this file are subject to the Rapid Transform Public License 
 * Version 1.1 (the "License"); you may not use this file except in 
 * compliance with the License. You may obtain a copy of the License at 
 * http://www.rapidtransform.com/license.html
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is Rapid PLM Open Source.
 * 
 * The Initial Developer of the Original Code is Rapid Transform, Inc.
 * Portions created by Rapid Transform are Copyright (C) 2005 Rapid Transform, Inc. 
 * All Rights Reserved.
 * 
 * Contributor(s): ______________________________________.
 */
package com.rapidtransform.part.service.impl;

import java.util.Collection;
import java.util.Date;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.dao.DataAccessException;
import org.springframework.transaction.annotation.Transactional;

import com.rapidtransform.core.ConveloException;
import com.rapidtransform.core.model.Relation;
import com.rapidtransform.core.model.namespace.QName;
import com.rapidtransform.core.service.impl.MetaCRUDServiceImpl;
import com.rapidtransform.doc.model.Document;
import com.rapidtransform.doc.model.Documented;
import com.rapidtransform.part.dao.ItemDAO;
import com.rapidtransform.part.dao.ItemInstanceDAO;
import com.rapidtransform.part.dao.ItemVersionDAO;
import com.rapidtransform.part.dao.ItemViewDAO;
import com.rapidtransform.part.model.Assembly;
import com.rapidtransform.part.model.CollectionView;
import com.rapidtransform.part.model.Item;
import com.rapidtransform.part.model.ItemInstance;
import com.rapidtransform.part.model.ItemVersion;
import com.rapidtransform.part.model.ItemView;
import com.rapidtransform.part.model.QuantifiedInstance;
import com.rapidtransform.part.model.SelectedInstance;
import com.rapidtransform.part.model.SingleInstance;
import com.rapidtransform.part.model.SpecifiedInstance;
import com.rapidtransform.part.service.ItemService;
import com.rapidtransform.spec.model.Specification;
import com.rapidtransform.spec.model.Specified;

/**
 * This service implementation provides find and CRUD methods for Items, ItemVersions, ItemViews,
 * and ItemInstances.  This class provides the transaction wrapper for all operations and can be extended
 * to provide other custom extensions, such as starting workflow operations or validating business rules
 * prior to performing the action.  The service class is a logical object to create new Aspects or Interceptors
 * which do the work for the business rules, workflow, or any other custom operations.
 * 
 * Using the Spring Framework and Hibernate, it is easy to configure and implement a custom Aspect or interceptors,
 * and apply them to any class or method via configuration.
 * 
 * @author S. Werner
 *
 */
public class ItemServiceImpl extends MetaCRUDServiceImpl<ItemVersion> implements ItemService {
	private static Log log = LogFactory.getLog(ItemServiceImpl.class);
	private ItemDAO itemDao;
	private ItemVersionDAO itemVersionDao;
	private ItemViewDAO itemViewDao;
	private ItemInstanceDAO itemInstanceDao;
	
	public ItemServiceImpl() {
		super(ItemVersion.class);
	}
	
	@Override
	public Log getLog() {
		return log;
	}
	
	public void setItemVersionDAO(ItemVersionDAO itemVersionDao) {
		this.itemVersionDao = itemVersionDao;
		setDao(itemVersionDao);
	}
	
	public void setItemDAO(ItemDAO itemDao) {
		this.itemDao = itemDao;
	}

	public void setItemInstanceDAO(ItemInstanceDAO itemInstanceDao) {
		this.itemInstanceDao = itemInstanceDao;
	}

	public void setItemViewDAO(ItemViewDAO itemViewDao) {
		this.itemViewDao = itemViewDao;
	}
	
	/**
	 * Returns the Item with the specified Id.
	 */
	public Item findItem(String itemId) throws ConveloException, DataAccessException {
		return itemDao.findById(Long.valueOf(itemId));
	}

	/**
	 * Returns a paged list of Items.
	 */
	public List findItems(Integer resultsPerPage) throws ConveloException, DataAccessException {
		return itemDao.findAll();
	}
	
	/**
	 * Creates a new Item and returns the initial version (ItemVersion) for the Item.  The Item
	 * information can be accessed via the initial ItemVersion.
	 * 
	 * @return
	 * @throws ConveloException
	 */
	@Override
	public ItemVersion create() throws ConveloException {
		Item item = new Item();
		ItemVersion initialVersion = item.createNewVersion();
		initialVersion.setCreatedDate(new Date());
		initialVersion.setCreatedBy(getSecurityService().getAuthenticatedUser());
		return initialVersion;
	}
	
	/**
	 * Creates a new version for the given item.  The ItemVersion is added to the list of versions
	 * contained in the item object.
	 * 
	 * @param item
	 * @return
	 */
	public ItemVersion revise(Item item) throws ConveloException {
		ItemVersion version = item.createNewVersion();
		return version;
	}


	@Transactional()
	public ItemView saveItemView(ItemView itemView) throws ConveloException, DataAccessException {
		return itemViewDao.save(itemView);
	}

	@Transactional()
	public Assembly saveAssembly(Assembly assembly) throws ConveloException, DataAccessException {
		return (Assembly)saveItemView(assembly);
	}

	@Transactional()
	public CollectionView saveCollectionView(CollectionView collectionView) throws ConveloException, DataAccessException {
		return (CollectionView)saveItemView(collectionView);
	}

	@Transactional()
	public ItemInstance saveItemInstance(ItemInstance itemInstance) throws ConveloException, DataAccessException {
		return itemInstanceDao.save(itemInstance);
	}
	
	@Transactional()
	public SingleInstance saveSingleInstance(SingleInstance singleInstance) throws ConveloException, DataAccessException {
		return (SingleInstance)saveItemInstance(singleInstance);
	}

	@Transactional()
	public QuantifiedInstance saveQuantifiedInstance(QuantifiedInstance quantifiedInstance)
	 	throws ConveloException, DataAccessException {
		
		return (QuantifiedInstance)saveItemInstance(quantifiedInstance);
	}

	@Transactional()
	public SelectedInstance saveSelectedInstance(SelectedInstance selectedInstance) 
	 	throws ConveloException, DataAccessException {
		
		return (SelectedInstance)saveItemInstance(selectedInstance);
	}

	@Transactional()
	public SpecifiedInstance saveSpecifiedInstance(SpecifiedInstance specifiedInstance)
		throws ConveloException, DataAccessException {
		
		return (SpecifiedInstance)saveItemInstance(specifiedInstance);
	}

	/* (non-Javadoc)
	 * @see com.rapidtransform.spec.service.SpecifiedService#addSpecification(com.rapidtransform.spec.model.Specified, com.rapidtransform.spec.model.Specification, java.lang.String)
	 */
	public Relation<ItemVersion, Specification> addSpecification(Specified source, Specification target, String relationType) throws ConveloException {
		Relation<ItemVersion, Specification> rel = null;
		if (source instanceof ItemVersion) {
			rel = createRelation((ItemVersion)source, target, new QName(relationType));
		}
		else {
			log.error("Wrong type passed to createSpecification; expecting ItemVersion, received "+source.getClass().getName());
			throw new ConveloException("Wrong type passed to createSpecification; expecting ItemVersion, received "+source.getClass().getName());
		}
		return rel;
	}

	/* (non-Javadoc)
	 * @see com.rapidtransform.spec.service.SpecifiedService#getSpecifications(java.lang.String, java.lang.String)
	 */
	public Collection<Specification> getSpecifications(String relationQname, String specifiedId) throws ConveloException {
		Collection<Specification> specs = null;
		specs = itemVersionDao.getSpecifications(new QName(relationQname), specifiedId);
		return specs;
	}

	/* (non-Javadoc)
	 * @see com.rapidtransform.doc.service.DocumentedService#addDocument(com.rapidtransform.doc.model.Documented, com.rapidtransform.doc.model.Document, java.lang.String)
	 */
	public Relation<ItemVersion, Document> addDocument(Documented source, Document target, String relationType) throws ConveloException {
		Relation<ItemVersion, Document> rel = null;
		if (source instanceof ItemVersion) {
			rel = createRelation((ItemVersion)source, target, new QName(relationType));
		}
		else {
			log.error("Wrong type passed to addDocument; expecting Product, received "+source.getClass().getName());
			throw new ConveloException("Wrong type passed to addDocument; expecting Product, received "+source.getClass().getName());
		}
		return rel;
	}

	/* (non-Javadoc)
	 * @see com.rapidtransform.doc.service.DocumentedService#getDocuments(java.lang.String, java.lang.String)
	 */
	public Collection<Document> getDocuments(String relationQname, String documentedId) throws ConveloException {
		Collection<Document> docs = null;
		docs = itemVersionDao.getDocuments(new QName(relationQname), documentedId);
		return docs;
	}


	
	
}
